****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_qrpe_attrition.sas  
*
* Created (mm/dd/yyyy): 03/01/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create the attrition table.                                        
*   
*  Program inputs:                                                                                   
*  	-Many datasets created by the ms_qrpenum macro
* 
*  Program outputs:                                                                                                                                       
*  	- The MSOC.&RUNID._Attrition output table
* 
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_qrpe_attrition();

%put =====> MACRO CALLED: ms_qrpe_attrition v1.0;

    /*****************/
    /* Utility macro */
    /*****************/

    %macro attrition(file=,ToExcl=);
	    proc sql noprint;
		    select count(distinct PatId),
				   count(distinct EncounterID),
				   count(distinct Facility_code)

					into :NumPatIDToExcl, :NumEncIDToExcl, :NumFacToExcl
		    from &file.
		    where not (&ToExcl.);
	    quit;
	    %put &NumPatIDToExcl. &NumEncIDToExcl. &NumFacToExcl.;

	    data &file.;
		    set &file.;
		    where not (&ToExcl.);
	    run;

	    %ISDATA(dataset=_Attrition);
	    %IF %EVAL(&NOBS.>=1) %THEN %DO;
	        data _Attrition;
		        set _Attrition end=eof;
		        output;
		        if eof then do;
		            Level=input("&level.",best.);
		            NumPts=input("&NumPatIDToExcl.",best.);
		            NumEnc=input("&NumEncIDToExcl.",best.);
		            NumFac=input("&NumFacToExcl.",best.);
		            call symputx("level",&level.+1);
		            output;
		        end;
	        run;
	    %END;
	    %ELSE %DO;
	        data _Attrition;
		        format level best. NumPts NumEnc NumFac comma10.;
		        Level=input("&level.",best.);
		        NumPts=input("&NumPatIDToExcl.",best10.);
		        NumEnc=input("&NumEncIDToExcl.",best10.);
		        NumFac=input("&NumFacToExcl.",best10.);
		        call symputx("level",&level.+1);
		        output;
	        run;
	    %END;
	    %put %eval(&level.-1) &NumPatIDToExcl. &NumEncIDToExcl. &NumFacToExcl.;
    %mend; 

/*********************************************/
/* Prepare Raw Encounter data for waterfall  */
/*********************************************/

    *non-missing birth date/sex - mutliple demog entries (should be zero - defensive coding);
    data _Demogsmissing;
    set indata.&Demtable.;
    where missing(BIRTH_DATE) or missing(Sex);
    keep PatId;
    run;

    %ms_delpatients(datafile=_Demogsmissing,
                    ptsfile=&PTSTOEXCLUDE.,
                    Outfile=_Demogsmissing);

    proc sort nodupkey data=_Demogsmissing;
    	by patid;
    run;
	proc sort nodupkey data=indata.&Demtable.(keep=patId BIRTH_DATE Sex Race Hispanic) out=_demogs dupout=_MultiDemogs(keep=PatId);
   		by PatId;
    run;
    proc sort nodupkey data=_MultiDemogs;
    	by PatId;
    run;

    data _encattr;
	    merge encextract(in=a)
	          _demogs(in=b)
	          _Demogsmissing(in=c)
	          _MultiDemogs(in=d);
	    by PatId;
	    if a and b and not c and not d;

		/*Remove encounters based on value of TRUNCYN*/
		%if "&truncyn" = "Y" %then %do;
			if ddate >&enddate. then delete;
		%end;

		MeetFacility = 1;
    run;

	* Restrict to desired facilities;
	%isdata(dataset=infolder.&FACILITYCODES.);;
	%if %eval(&nobs.>0) %then %do;
		proc sort data=_encattr;
			by facility_code;
		run;
		proc sort data=infolder.&FACILITYCODES. out=_facilities nodupkey;
			by facility_code;
		run;

		data _encattr;
			merge _encattr (in=a)
				  _facilities (in=b);
			by facility_code;
			if a;
			if a and not b then MeetFacility = 0;
		run;
	%end;

    data _encattr;
    	set _encattr;
		all=1;
		    
		*demographic criteria;
		MeetSex=1;
		MeetRace=1;
		MeetHispanic=1;
		if upcase(sex) not in (&DemogSex.) then MeetSex = 0;
		if upcase(Race) not in (&DemogRace.) then MeetRace = 0;
		if upcase(Hispanic) not in (&DemogHispanic.) then MeetHispanic = 0;

		*Caresetting;
		MeetCaresetting=1;
		if upcase(EncType) not in (&EncCaresetting.) then MeetCaresetting = 0;

    run;

    /** CALCULATE AGE and AGE STRATA SPECIFIED BY MSOC **/
    %ms_agestrat(infile=_encattr,
                 outfile=_agegroups,
                 startdt=birth_date,
                 enddt=adate,
                 timestrat=&agestrat.);

	data _agegroups;
		set _agegroups;
		keep patid encounterID agegroup;
		if agegroup ne '';
	run;

	proc sort data=_agegroups nodupkey;
		by patid encounterID;
	run;
	proc sort data=_encattr;
		by patid encounterID;
	run;
   
    data _keep;
    merge _encattr(in=a)
          _agegroups(in=b)
		  _pov1 (in=c)
		  _pov3 (in=d);
    by PatId encounterID;
    if a;

   		if b then AgeGroup="Yes";
    	else AgeGroup="";

		if c then Exposure=1;
			else Exposure=0;

		if d then ExlInc=1;
			else ExlInc=0;
    run;

    *Reset cumulative;
    proc datasets library=work nowarn nolist;
    delete _attrition;
    quit;

    %global level;
    %let level=1;

    /*1*/	%attrition(file=_keep,ToExcl=all=0);
    /*2*/	%attrition(file=_keep,ToExcl=MeetFacility=0);
    /*3*/	%attrition(file=_keep,ToExcl=MeetCaresetting=0);
    /*4*/	%attrition(file=_keep,ToExcl=AgeGroup='');
   	/*5*/	%attrition(file=_keep,ToExcl=MeetSex=0);
    /*6*/	%attrition(file=_keep,ToExcl=MeetHispanic=0);
	/*7*/	%attrition(file=_keep,ToExcl=MeetRace=0);
	/*8*/	%attrition(file=_keep,ToExcl=ExlInc=0);
	/*9*/	%attrition(file=_keep,ToExcl=Exposure=0);

	/**********************************/
	/* Calculate Exclusions variables */
	/**********************************/

    data _Attrition;
	    retain group level descr NumPts NumEnc NumFac ExcludedPts ExcludedEnc ExcludedFac;
	    format group $30. descr $500.;
	    set _Attrition;
	    group="&ITGROUP.";

	    format ;
	    LastRemainPts=lag(NumPts);
	    LastRemainEnc=lag(NumEnc);
	    LastRemainFac=lag(NumFac);

	    ExcludedPts=LastRemainPts-NumPts;
	    ExcludedEnc=LastRemainEnc-NumEnc;
	    ExcludedFac=LastRemainFac-NumFac;
	    
        if level=1 then descr="Encounter during the query period";
        if level=2 then descr="Encounter restricted by facilities during the query period";
        if level=3 then descr="Encounters restricted by encounter type";
        if level=4 then descr="Encounters restricted by age group";
        if level=5 then descr="Encounters restricted by specified sex";
        if level=6 then descr="Encounters restricted by specified ethnicity";
		if level=7 then descr="Encounters restricted by specified race";
        if level=8 then descr="Encounters restricted by additional inclusion/exclusion criteria";
		if level=9 then descr="Encounters with exposure of interest during the query period";

	    rename NumPts=RemainingPts NumEnc=RemainingEnc NumFac=RemainingFac;
	    drop LastRemainPts LastRemainEnc LastRemainFac;
    run;

	*Store attrition to MSOC;
    %IF %EVAL(&group.=1) %THEN %DO;
        data MSOC.&RUNID._Attrition;
        retain group level descr RemainingEnc ExcludedEnc RemainingPts ExcludedPts RemainingFac ExcludedFac;
        set _Attrition;
        run;
    %END;
    %ELSE %DO;
        proc append base=MSOC.&RUNID._Attrition 
                    data=_Attrition force;
        run;
    %END;

%put NOTE: ******** END OF MACRO: ms_qrpe_attrition v1.0 ********;

%mend ms_qrpe_attrition;
