****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_qrpe_processinputfiles.sas  
*
* Created (mm/dd/yyyy): 02/01/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will process the program input files variables.                                        
*   
*  Program inputs:  
*	- a dataset containing the cohort definition codes
*	- a dataset containing inclusion/exclusion codes
*	- a dataset containing the monitoring file period
*	- a dataset containing the cohort definition parameters
*             	                        
*  Program outputs:                                                                                                                                       
*	- datasets with formatted input files variables
*   - _diag, _proc, _inpatndc, _outpatndc, and _trans that will be used for code extraction
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_qrpe_processinputfiles();

%put =====> MACRO CALLED: ms_qrpe_processinputfiles v1.0;

/**********************************************/
/* Diagnosis, Procedure, and Dispensing codes */
/**********************************************/

*Cohortcodes;	
	data &COHORTCODES.;
	set infolder.&COHORTCODES.;
    Code=compress(Code,'. ');
	run;	

*InclusionCodes;
    %IF %SYSFUNC(exist(infolder.&INCLUSIONCODES.))=1 %THEN %DO;
	    data &INCLUSIONCODES.;
		    set infolder.&INCLUSIONCODES.;

		    if CondInclusion=1 then do;
		        INDEX="INC";
		    end;
		    if CondInclusion=0 then do;
		        INDEX="EXC";
		    end;

		    FUP="NOT";

			*Defensive coding: upcase condlevel and subcondlevel;
			condlevel=upcase(condlevel);
			subcondlevel=upcase(subcondlevel);
			if missing(codedays) then codedays = 1;
	    run;

		*Ensure only 1 codedays value per group condlevel subcondlevel;
		proc sql noprint;
			create table _codedays_check as 
			select distinct group
				 , condlevel
				 , subcondlevel
			from &INCLUSIONCODES.
			group by group, condlevel, subcondlevel
			having count(distinct codedays) > 1
			;
		quit;

		%isdata(dataset=_codedays_check);
		%if %eval(&nobs.>0) %then %do;
			%put WARNING: Different CODEDAYS value specified within the same SUBCOND. Check Inclusion/Exclusion code file integrity;
		%end;

		*Ensure only 1 gapdays value per group condlevel subcondlevel;
		proc sql noprint;
			create table _gapdays_check as 
			select distinct group
				 , condlevel
				 , subcondlevel
			from &INCLUSIONCODES.
			group by group, condlevel, subcondlevel
			having count(distinct gapdays) > 1
			;
		quit;

		%isdata(dataset=_gapdays_check);
		%if %eval(&nobs.>0) %then %do;
			%put WARNING: Different GAPDAYS value specified within the same SUBCOND. Check Inclusion/Exclusion code file integrity;
		%end;

		proc sort data=&INCLUSIONCODES. out=&INCLUSIONCODES.;
			by group CondLevel SubCondLevel;
		run;

		*Create numeric Condlevel and SubCondlevel variables for looping;
		data &INCLUSIONCODES. (drop=l_subcond l_cond);
		set &INCLUSIONCODES.;
			format cond subcond 8.;
			by group CondLevel;

			l_cond=lag(CondLevel);
			l_subcond=lag(SubCondLevel);

			if first.group then do;
				cond=1;
				call missing(l_cond);
			end;
			else do;
				if CondLevel=l_cond then do;
					cond=cond;
				end;
				else if CondLevel^=l_cond then do;
					cond=cond+1;
				end;
			end;
			*Create numeric subcondlevel variable;
			if first.CondLevel then do;
				subcond=1;
				call missing(l_subcond);
			end;
			else do;
				if SubCondLevel=l_subcond then do;
					subcond=subcond;
				end;
				else if SubCondLevel^=l_subcond then do;
					subcond=subcond+1;
				end;
			end;
			retain cond subcond;
			run;

		*Append inclusion and Exclusion codes to work.&COHORTCODES.;
		data &COHORTCODES.;
		    set &COHORTCODES.
		        &INCLUSIONCODES.;
	  	 	Code=compress(Code,'. ');
	    run;
    %END;

/*******************/
/* Monitoring File */
/*******************/

    %IF %SYSFUNC(exist(infolder.&Monitoringfile.))=1 %THEN %DO;
        *Default period selection;
        data _null_;
			call symputx("PERIODIDSTART",1);
			call symputx("PERIODIDEND",1);   
        run;

    	*Taking min startfollowup and max enddate;
        proc means data=infolder.&Monitoringfile. nway noprint;
        var startfollowup enddate ;
        where &PERIODIDSTART. <= PeriodId  <= &PERIODIDEND.;
        output out=&Monitoringfile._agg min(startfollowup)=startfollowup
                                    max(enddate)=enddate;
        run; 

		data &Monitoringfile.; 
			set infolder.&Monitoringfile.;
		run;
        
        data &Monitoringfile._agg;
	        set &Monitoringfile._agg;    
	        call symputx("startfollowup",startfollowup);
	        call symputx("enddate",enddate);    
        run;

        %put &startfollowup. &enddate. &numperiodid.;
    %END;

/*********************************/
/* Preprare Raw extraction files */
/*********************************/
	%ISDATA(dataset=infolder.&COVARIATECODES.);	    
    %IF %EVAL(&NOBS.>0) %THEN %DO;
	   	*Select covariates codes (not combination of covatiates);
		*to prevent w a r n i n g when covariatecodes contains >11 length for CC codes, reset length of code*;
		data CovarCodes;
			length code $11;
			format code $11.;
			set infolder.&COVARIATECODES.(rename=code=code1);
			where codecat not in ("CC");
			code=code1;
			drop code1;
 		run;
	%END;
	%ELSE %DO;
		data CovarCodes;
			studyname = "";
			CovarNum = 0; 
	        Code = "";
			CodeCat = "";
			Codetype = "";
			CARESETTINGPRINCIPAL = "";
			Combination = .;
			CovFrom=.;
			CovTo=.;
			covartype="";
			Padmit="";
        run;
	%END;

	proc sort nodupkey data=CovarCodes out=StudyNames(keep=studyname CovarNum covartype);
    	by CovarNum;
    run;

	*Combine COHORTCODES and COVARCODES codes to pull data from CDM only once;
	data _diag
         _proc
	     _inpatndc
		 _trans
		 /*_outpatndc*/;

	    set &COHORTCODES.
	        CovarCodes;

	    Code=upcase(compress(Code,'. '));
	    length=length(code);
	    CodeCat=upcase(CodeCat);

	    *if CodeCat in('DR') then output _outpatndc;
	    if CodeCat in('IR') then output _inpatndc;
	    if CodeCat in('DX') then output _diag;
	    if CodeCat in('PX') then output _proc;
	    if CodeCat in('TR') then output _trans;
		keep Code CodeCat Codetype;
    run;

	*Creating the number of groups to create cohorts;
    %isdata(dataset=infolder.&COHORTFILE.);    
    data _null_;
   		call symputx("Ngroup","&nobs.");
    run;

	*Transfusion lookup table;
	%isdata(dataset=infolder.&TRANSCODE_TYPE_LOOKUP.);
	%if %eval(&nobs.>0) %then %do;
		data transcodes;
			set infolder.&TRANSCODE_TYPE_LOOKUP.;
			keep codetype clincode requirement;
			rename clincode =code;
		run;
	%end;
	%else %do;
		data transcodes;
			format requirement $15. codecat $2. code $15.;
			requirement = '';
			codecat = '';
			code = '';
			stop;
		run;
	%end;

	/** HOI **/
	proc sort nodupkey data= infolder.&COHORTCODES. out=HOILIST;
		by FUP;
		where FUP =:"HOI";
	run;

	%ISDATA(dataset=HOILIST);	
	%IF %EVAL(&NOBS.>0) %THEN %DO;
		proc sql noprint;
			select count(FUP) into :NUM_HOI 
			from HOILIST;
		quit;
	%END;
	%else %do;
		%let NUM_HOI = 0;
	%end;

	data _null_;
		call symputx("NUM_HOI",&NUM_HOI.);
	run;
	%put &NUM_HOI.;

	%put NOTE: ******** END OF MACRO: ms_qrpe_processinputfiles v1.0 ********;

%mend ms_qrpe_processinputfiles;
